#!/usr/bin/python3


import argparse
import gettext
import os
import sys
from argparse import RawTextHelpFormatter

import astra_events
from astra_admin_events.tree import Tree

gettext.bindtextdomain('astra_admin_events',
                       '/usr/share/locale')
gettext.textdomain('astra_admin_events')
_ = gettext.gettext


def get_args():
    description = _(
        "Console event logging setup utility\n to use --properties look at all keys and values in /usr/share/syslog-ng-mod-astra/event-settings directory or use astra-admin-events event_id\n write all arguments at the end\n all options for --priority is below: \n    facilities:\n        'kern',\n        'user',\n        'mail',\n        'daemon',\n        'auth',\n        'syslog',\n        'lpr',\n        'news',\n        'uucp',\n        'cron',\n        'authpriv',\n        'ftp',\n        'ntp',\n        'security',\n        'console',\n        'solaris-cron',\n        'local0',\n        'local1',\n        'local2',\n        'local3',\n        'local4',\n        'local5',\n        'local6',\n        'local7',\n    priorities: \n        'emergency',\n        'critical',\n        'error',\n        'informational',\n        'emerg',\n        'alert',\n        'crit',\n        'err',\n        'warning',\n        'notice',\n        'info',\n        'debug'")
    parser = argparse.ArgumentParser(description=description, formatter_class=RawTextHelpFormatter, add_help=True)

    parser.add_argument('--enable', action='store_true', help=_('enable event'))
    parser.add_argument('--disable', action='store_true', help=_('disable event'))
    parser.add_argument('--properties', action='store_true',
                        help=_(
                            "enable/disable properties of event ('event-chmod' 'extend-message-params' 'exe' '{'key':'.auditd.exe', 'enabled' :'1'}')"))
    parser.add_argument('--export', action='store_true',
                        help=_("configuration export (astra-admin-events '/way/file_name' --export)"))
    parser.add_argument('--import', action='store_true',
                        help=_("configuration import  (astra-admin-events '/way/file_name.tar.gz' --import)"))
    parser.add_argument('--priority',
                        help=_("change the priority of event (astra-admin-events event_id --priority critical)"))
    parser.add_argument('--facility', help=_("change facility of event (astra-admin-events event_id --facility user)"))
    return parser


def main():
    if os.geteuid() != 0:
        print(_("Use sudo"))
        sys.exit(1)
    tree = Tree()
    parser = get_args()
    if '--export' in sys.argv:
        tree.export_config(sys.argv[1])
        sys.exit(0)
    if '--import' in sys.argv:
        tree.import_config(sys.argv[1])
        sys.exit(0)
    if '--help' in sys.argv or '-h' in sys.argv:
        parser.print_help()
        sys.exit(0)
    elif len(sys.argv) == 1:
        tree.tree_output('')
    elif '--enable' in sys.argv:
        tree.enabling(sys.argv[1:-1])
    elif '--disable' in sys.argv:
        tree.disable(sys.argv[1:-1])
    elif '--properties' not in sys.argv and '--enable' not in sys.argv and '--disable' not in sys.argv \
            and '--priority' not in sys.argv and '--facility' not in sys.argv:
        for dir_name in os.listdir('/usr/share/syslog-ng-mod-astra/event-settings'):
            if sys.argv[1] in os.listdir(f'/usr/share/syslog-ng-mod-astra/event-settings/{dir_name}') and sys.argv[1] \
                    in astra_events.get_event_ids():
                tree.nested_tree(f'/usr/share/syslog-ng-mod-astra/event-settings/{dir_name}/{sys.argv[1]}')
        if sys.argv[1] in astra_events.get_group_ids():
            tree.tree_output(sys.argv[1])
    elif '--properties' in sys.argv:
        if len(sys.argv) != 6:
            print('not enough arguments')
            sys.exit(1)
        else:
            tree.changing_event_properties(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif '--priority' in sys.argv or '--facility' in sys.argv:
        if '--priority' in sys.argv:
            tree.change_priority(sys.argv[1], sys.argv[sys.argv.index('--priority') + 1])
        if '--facility' in sys.argv:
            tree.change_facility(sys.argv[1], sys.argv[sys.argv.index('--facility') + 1])


if __name__ == '__main__':
    main()
