#!/usr/bin/python3
import gettext
import json
import locale
import os
import os.path
import pydoc
import re
import sys
import platform

import astra_events

gettext.bindtextdomain('astra_admin_events',
                       '/usr/share/locale')
gettext.textdomain('astra_admin_events')
_ = gettext.gettext

if locale.getlocale()[0] == 'ru_RU':
    lang_key = 'name_ru'
else:
    lang_key = 'name_en'


class Tree:
    __arch = platform.machine()
    __audit_rules = {
        "x86_64": "-a always,exit -F arch=x86_64 -F path={0} -S open,openat -k {0}",
        "aarch64": "-a always,exit -F arch=aarch64 -F path={0} -S openat -k {0}",
        "armv7l": "-a always,exit -F arch=armv7l -F path={0} -S open,openat -k {0}",
    }
    def __init__(self):
        self.dir_path = '/usr/share/syslog-ng-mod-astra/event-settings'
        self.output_page = []

    def tree_output(self, one_name_file):
        endc = '\033[0m'
        grey = '\033[90m'
        list_of_events = os.listdir(self.dir_path)
        for file_name in list_of_events:
            if one_name_file and one_name_file != 'foo':
                file_name = one_name_file
            with open(self.dir_path + '/' + file_name + '/' + file_name, 'r') as file_:
                json_type_of_file = json.load(file_)
                self.output_page.append(json_type_of_file['group-params'][lang_key] + ' ' + grey + file_name + endc)
                for event_list in os.walk(self.dir_path + '/' + file_name):
                    for event in event_list[2]:
                        if event in astra_events.get_event_ids():
                            with open(self.dir_path + '/' + file_name + '/' + event, 'r') as file:
                                json_type_of_file = json.load(file)
                                spaces = len(max(event_list[2], key=len)) + 8
                                self.output_page.append(
                                    event + (spaces - len(event)) * ' ' + str(
                                        astra_events.is_event_enabled(event)) + ' ' +
                                    grey + json_type_of_file['message-params'][lang_key] + endc)
            self.output_page.append(' ')
            if one_name_file and one_name_file != 'foo':
                break
        if one_name_file and one_name_file != 'foo':
            print('\n'.join(self.output_page))
        elif one_name_file == 'foo':
            pass
        else:
            pydoc.pipepager('\n'.join(self.output_page), cmd='less -R')

    def enabling(self, event_ids):
        for event_id in event_ids:
            event_id = self.check_event_id(event_id)
            if event_id in astra_events.get_group_ids():
                self.enabling_group(event_id)
                self.tree_output(event_id)
            else:
                astra_events.enable_event(event_id)
                self.tree_output('foo')
                print(self.show_event_after_enabling(event_id))
        self.reboot_daemons()

    def disable(self, event_ids):
        for event_id in event_ids:
            event_id = self.check_event_id(event_id)
            if event_id in astra_events.get_group_ids():
                self.disable_group(event_id)
                self.tree_output(event_id)
            else:
                astra_events.disable_event(event_id)
                self.tree_output('foo')
                print(self.show_event_after_enabling(event_id))
        self.reboot_daemons()

    def show_event_after_enabling(self, event_id) -> str:
        r = re.compile(event_id)
        needed_event_string = set(list(filter(r.match, self.output_page)))
        return '\n'.join(needed_event_string)

    @staticmethod
    def nested_tree(way):
        with open(way, 'r') as file:
            json_thing = json.load(file)
            with open('/etc/fly-admin-events/eventsdictionary.json', 'r') as file_nested_translations:
                ru_name = json.load(file_nested_translations)

                keys_dict = json_thing['extend-message-params']

                keys_dict = {
                    k + ' (' + ru_name[k]['ru'] + ')' if k in list(ru_name.keys()) and 'ru' in ru_name[k] else k: v for
                    k, v in
                    keys_dict.items()}
                json_thing['extend-message-params'] = keys_dict

                first_level_keys = json_thing
                first_level_keys = {k + ' ' + ru_name[k]['ru'] if 'ru' in ru_name[k].keys() else k: v for k, v in
                                    first_level_keys.items()}
                json_thing = first_level_keys

                pydoc.pipepager(json.dumps(json_thing, sort_keys=True, indent=4, ensure_ascii=False).replace(
                    '{', '').replace(
                    '}', '').replace('"', '').replace(',', '').replace('[', '').replace(']', ''), cmd='less -R')

    @staticmethod
    def enabling_group(group_id):
        events_to_enabled = os.listdir(f'/usr/share/syslog-ng-mod-astra/event-settings/{group_id}')
        for event_id in events_to_enabled:
            if event_id in astra_events.get_event_ids():
                astra_events.enable_event(event_id)

    @staticmethod
    def disable_group(group_id):
        events_to_enabled = os.listdir(f'/usr/share/syslog-ng-mod-astra/event-settings/{group_id}')
        for event_id in events_to_enabled:
            if event_id in astra_events.get_event_ids():
                astra_events.disable_event(event_id)

    def changing_event_properties(self, event_id: str, section: str,
                                  parameter: str, value: str):
        event_id = self.check_event_id(event_id)
        try:
            value_check_result = self.check_value_correct(value)
            formatted_way_to_file = value_check_result[0]
            value = value_check_result[1]
        except Exception as e:
            print("Check values")
            sys.exit()
        if event_id == 'modifying-file' or event_id == 'modifying-directory' or event_id == 'file-removal' \
                or event_id == 'file-opened':
            self.special_properties(event_id, section, parameter, value, formatted_way_to_file)
        else:
            astra_events.set_event_param(event_id, section, parameter, value)
        self.reboot_daemons()

    def change_priority(self, event_id, priority_key):
        event_id = self.check_event_id(event_id)
        astra_events.set_event_param(event_id, None, "message-syslog-priority", f'"{priority_key}"')
        self.reboot_daemons()
        print(_(f'For {event_id} priority: {priority_key}'))

    def change_facility(self, event_id, facility):
        event_id = self.check_event_id(event_id)
        astra_events.set_event_param(event_id, None, "message-syslog-facility", f'"{facility}"')
        self.reboot_daemons()
        print(_(f'For {event_id} facility: {facility}'))

    @staticmethod
    def export_config(way_to_tar):
        os.system(f'tar -czvf {way_to_tar}.tar.gz /etc/syslog-ng/conf.d /etc/audit/rules.d/astra-syslog.rules '
                  f'/etc/logrotate.d/syslog-ng-mod-astra /etc/astra-syslog.conf /root/.config/astra-event-watcher.notifyrc >/dev/null 2>&1')
        print(_('Configuration exported'))

    def import_config(self, way_to_tar):
        os.system(f'tar -xf {way_to_tar} -C /')
        self.reboot_daemons()
        print(_('Configuration imported'))

    def special_properties(self, event_id: str, section: str,
                           parameter: str, value: str, formatted_way_to_file: list):
        event_id = self.check_event_id(event_id)
        rule = 'wa'
        if event_id != 'file-opened':
            value_for_message_audit_rules = list(
                map(lambda file_dir: "-w {0} -p {1} -k {0}" \
                    .format(file_dir.replace(" ", "\\ "), rule),
                    formatted_way_to_file))
        else:
            value_for_message_audit_rules = list(
                map(lambda file_dir:
                    Tree.__audit_rules.get(Tree.__arch, "-a always,exit -F arch=b64 -F path={0} -S open,openat -k {0}") \
                    .format(file_dir.replace(" ", "\\ ")),
                    formatted_way_to_file))

        final_value = json.dumps(value_for_message_audit_rules, ensure_ascii=False)

        astra_events.set_event_param(event_id, section, parameter, value)
        astra_events.set_event_param(event_id, None, "message-audit-rules", final_value)

    def check_value_correct(self, value):
        formatted_way_to_file = []
        try:
            data = json.loads(value)
        except Exception as e:
            print(e)
            return
        if isinstance(data, str):
            formatted_way_to_file.append(data)
        elif isinstance(data, list):
            formatted_way_to_file.extend(data)
        elif isinstance(data, dict):
            formatted_way_to_file.extend(data)
        else:
            print("Invalid value {}".format(value))
            return
        return formatted_way_to_file, value

    @staticmethod
    def reboot_daemons():
        os.system('systemctl restart syslog-ng.service')
        os.system('systemctl restart auditd')

    @staticmethod
    def check_event_id(event_id: str):
        if event_id in astra_events.get_event_ids() or event_id in astra_events.get_group_ids():
            return event_id
        else:
            event_id = event_id.replace('_', '-')
            if event_id in astra_events.get_event_ids() or event_id in astra_events.get_group_ids():
                return event_id
            else:
                print("Check event id")
                sys.exit(1)
